function keyframes({
  keyframes,
  easing,
  easingsByKeyframe = {}
}) {
  'worklet';

  validateKeyframes(keyframes);
  return t => {
    'worklet';

    for (let keyframeIndex = 0; keyframeIndex < keyframes.length - 1; keyframeIndex++) {
      const currentStepEndTime = (keyframes[keyframeIndex + 1] - keyframes[0]) / (keyframes[keyframes.length - 1] - keyframes[0]);

      if (t < currentStepEndTime) {
        var _easingsByKeyframe$ke;

        const currentStepStartTime = (keyframes[keyframeIndex] - keyframes[0]) / (keyframes[keyframes.length - 1] - keyframes[0]);
        const currentStepDuration = currentStepEndTime - currentStepStartTime;
        const currentRelativeRatio = (t - currentStepStartTime) / currentStepDuration;
        const currentEasing = (_easingsByKeyframe$ke = easingsByKeyframe[keyframes[keyframeIndex]]) !== null && _easingsByKeyframe$ke !== void 0 ? _easingsByKeyframe$ke : easing;
        return currentStepStartTime + currentEasing(currentRelativeRatio) * currentStepDuration;
      }
    }

    return t;
  };
}

function validateKeyframes(keyframes) {
  if (keyframes.length < 3) {
    throw new Error('keyframes must have at least 3 items');
  }

  for (let i = 0; i < keyframes.length; i++) {
    const currentKeyframe = keyframes[i];

    if (isNaN(currentKeyframe)) {
      throw new Error('A keyframe value must be a number');
    }

    if (i > 0) {
      const prevKeyframe = keyframes[i - 1];

      if (currentKeyframe <= prevKeyframe) {
        throw new Error("keyframes must be monotonically increasing: ".concat(JSON.stringify(keyframes)));
      }
    }
  }
}

const Easing = {
  keyframes
};
export { Easing };
//# sourceMappingURL=easing.js.map